const stackFrameMap = new WeakMap<Error, ReadonlyArray<any>>();

let prepareStackTraceWithSourceMap: (
  error: Error,
  frames: ReadonlyArray<any>
) => string;

export function withSourceMappedStack(error: Error): Error {
  return {
    name: error.name,
    message: error.message,
    stack: sourceMappedStackTrace(error)
  };
}

function sourceMappedStackTrace(error: Error): string | undefined {
  let frames = stackFrameMap.get(error);

  if (!frames) {
    // At this point there's no guarantee that anyone has actually retrieved the
    // stack on this error which means that our custom prepareStackTrace handler
    // hasn't run and as a result of that we don't have the native frames stored
    // in our weak map. In order to get around that we'll eagerly access the
    // stack, forcing our handler to run which should ensure that the native
    // frames are stored in our weak map.
    (error.stack || '').toString();
    frames = stackFrameMap.get(error);
  }

  if (!frames) {
    return error.stack;
  }

  return prepareStackTraceWithSourceMap(error, frames);
}
